@@ -34,12 +34,16 @@ the executable file might be covered by the GNU General Public License. */
 #define _IO_fscanf fscanf
 #endif
 
-int
-_IO_fscanf
 #ifdef __STDC__
-  (_IO_FILE *fp, const char* format, ...)
+int _IO_fscanf(_IO_FILE *, const char *, ... );
+
+int
+_IO_fscanf(_IO_FILE *fp, const char* format, ...)
 #else
-(fp, format, va_alist) _IO_FILE *fp; char *format; va_dcl
+int _IO_fscanf();
+
+int
+_IO_fscanf(fp, format, va_alist) _IO_FILE *fp; char *format; va_dcl
 #endif
 {
   int ret;
