@@ -61,6 +61,7 @@ static int netgroupmode = 0;
 static int inkeylen;
 static char *inkey = NULL;
 static struct passwd *stored_pwd = NULL;
+static char *nisdomain = NULL;
 
 /* This is the function that all the others are based on.
    The format of the password file is known only here.  */
@@ -280,6 +281,10 @@ _compat_getpwnam(register const char *name)
   static struct passwd *stored_pwd = NULL;
   register FILE *stream;
   register struct passwd *p;
+  char *netgroups = NULL;
+
+  if ('-' == name[0] || '+' == name[0])
+    return NULL;
 
   if (info == NULL)
     {
@@ -295,21 +300,58 @@ _compat_getpwnam(register const char *name)
     {
       if (NULL == stored_pwd)
 	stored_pwd = __nis_alloc_pwd_args();
-
-          /* Handle -@netgroup entries */
-      if ('-' == p->pw_name[0]
-          && '@' == p->pw_name[1]
-          && '\0' != p->pw_name[2]
-          && 1 == innetgr(&p->pw_name[2], NULL, name, NULL))
-        return NULL ;
       
-          /* Handle +@netgroup entries */
-      if ('+' == p->pw_name[0]
+          /* Handle -@netgroup entries */
+      if (('-' == p->pw_name[0]  || '+' == p->pw_name[0])
           && '@' == p->pw_name[1]
           && '\0' != p->pw_name[2])
-        {
-          if (1 == innetgr(&p->pw_name[2], NULL, name, NULL))
-            {
+	{
+	  char *s, *t;
+	  
+	  if (NULL == nisdomain)
+	    yp_get_default_domain(&nisdomain);
+	  if (! netgroups)
+	    {
+	      int status, outkeylen;
+	      int l = strlen(name);
+	      char namebuf[l + strlen(nisdomain) + 3];
+	      
+	      memcpy(namebuf, name, l);
+	      namebuf[l] = '.';
+	      strcpy(namebuf + l + 1, nisdomain);
+	      status = yp_match(nisdomain, "netgroup.byuser",
+				namebuf, l + strlen(nisdomain) + 1,
+				&netgroups, &outkeylen);
+	      if (status)
+		{
+		  namebuf[l+1] = '*';
+		  namebuf[l+2] = '\0';
+		  status = yp_match(nisdomain, "netgroup.byuser",
+				    namebuf, l + 2, &netgroups, &outkeylen);
+		  if (status)
+		    continue;
+		}
+	      
+	      netgroups[outkeylen] = ',';
+	      netgroups[outkeylen+1] = '\0';
+	    }
+	  
+	  t = s = netgroups;
+	  while (*s && *t)
+	    {
+	      t = p->pw_name + 2;
+	      while (*s != ',' && *t++ == *s++);
+	      while (*s++ != ',');
+	    }
+	  
+	  if (! *t)
+	    {
+	      if ('-' == p->pw_name[0])
+		{
+		  p = NULL;
+		  break;
+		}
+	      
 	      __nis_clear_pwd_args(stored_pwd);
 	      __nis_copy_pwd_args(p, stored_pwd, 0);
 	      p = __nis_getpwnam(name, info) ;
@@ -318,12 +360,15 @@ _compat_getpwnam(register const char *name)
             }
           continue;
         }
-        
+                      
           /*
            * Handle +user/-user entries.
            */
       if (('-' == p->pw_name[0]) && (0 == strcmp(p->pw_name + 1, name)))
-        return NULL;
+	{
+	  p = NULL;
+	  break;
+	}
       if (('+' == p->pw_name[0]) && (0 == strcmp(p->pw_name + 1, name)))
         {
 	  __nis_clear_pwd_args(stored_pwd);
@@ -349,6 +394,8 @@ _compat_getpwnam(register const char *name)
       if (0 == strcmp(p->pw_name, name))
         break;
     }
+  if (netgroups)
+    free(netgroups);
   (void) fclose(stream);
   return(p);
 }
@@ -360,6 +407,9 @@ _compat_getpwuid(register uid_t uid)
   static void *info_nis = NULL;
   static void *info = NULL;
   static struct passwd *stored_pwd = NULL;
+  struct passwd *nis_pwd = NULL;
+  char *netgroups = NULL;
+  int nis_failed = 0;
   register FILE *stream;
   register struct passwd *p;
 
@@ -379,33 +429,85 @@ _compat_getpwuid(register uid_t uid)
       if (NULL == stored_pwd)
 	stored_pwd = __nis_alloc_pwd_args();
 
-          /* Handle -@netgroup entries */
-      if ('-' == p->pw_name[0]
+          /* Handle -@netgroup/+@netgroup entries */
+      if (('-' == p->pw_name[0] || '+' == p->pw_name[0])
           && '@' == p->pw_name[1]
           && '\0' != p->pw_name[2])
-        {
-          p = __netgroup_getpwuid(&p->pw_name[2], uid, info);
-          if (NULL != p)
-            return NULL;
-	  continue;
-        }
+	{
+	  char *s, *t;
 
-          /* Handle +@netgroup entries */
-      if ('+' == p->pw_name[0]
-          && '@' == p->pw_name[1]
-          && '\0' != p->pw_name[2])
-        {
-	  __nis_clear_pwd_args(stored_pwd);
-	  __nis_copy_pwd_args(p, stored_pwd, 0);
-          p = __netgroup_getpwuid(&p->pw_name[2], uid, info);
-          if (NULL != p)
+	  if (nis_failed)
+	    continue; /* We failed once so do not try again */
+
+	  if (! netgroups)
 	    {
-	      __nis_copy_pwd_args(stored_pwd, p, 1);
-	      break;
+	      if (! info_nis)
+		{
+		  info_nis = __pwdalloc();
+		  if (NULL == info_nis)
+		    return(NULL);
+		}
+	      nis_pwd = __nis_getpwuid(uid, info_nis);
+	      if (! nis_pwd)
+		{
+		  nis_failed = 1;
+		  continue;
+		}
+	      else
+		{
+		  /* __nis_getpwuid already initialized nisdomain */
+		  int status, outkeylen;
+		  int l = strlen(nis_pwd->pw_name);
+		  char namebuf[l + strlen(nisdomain) + 3];
+
+		  memcpy(namebuf, nis_pwd->pw_name, l);
+		  namebuf[l] = '.';
+		  strcpy(namebuf + l + 1, nisdomain);
+		  status = yp_match(nisdomain, "netgroup.byuser",
+				    namebuf, l + strlen(nisdomain) + 1,
+				    &netgroups, &outkeylen);
+		  if (status)
+		    {
+		      namebuf[l+1] = '*';
+		      namebuf[l+2] = '\0';
+		      status = yp_match(nisdomain, "netgroup.byuser",
+					namebuf, l + 2,
+					&netgroups, &outkeylen);
+		      if (status)
+			{
+			  nis_failed = 1;
+			  continue;
+			}
+		    }
+
+		  netgroups[outkeylen] = ',';
+		  netgroups[outkeylen+1] = '\0';
+		}
 	    }
+
+	  t = s = netgroups;
+	  while (*s && *t)
+	    {
+	      t = p->pw_name + 2;
+	      while (*s != ',' && *t++ == *s++);
+	      while (*s++ != ',');
+	    }
+
+	  if (! *t)
+	    {
+	      if ('-' == p->pw_name[0])
+		{
+		  p = NULL;
+		  break;
+		}
+
+              __nis_copy_pwd_args(p, nis_pwd, 1);
+	      p = nis_pwd;
+	      break;
+            }
 	  continue;
-        }
-      
+	}
+
           /*
            * Handle +user/-user entries.
            */
@@ -413,20 +515,17 @@ _compat_getpwuid(register uid_t uid)
         {
           p = __nis_getpwnam(p->pw_name + 1, info);
           if (p && p->pw_uid == uid)
-            return NULL;
+	    {
+	      p = NULL;
+	      break;
+	    }
           continue;
         }
       if ('+' == p->pw_name[0] && '\0' != p->pw_name[1])
         {
 	  __nis_clear_pwd_args(stored_pwd);
 	  __nis_copy_pwd_args(p, stored_pwd, 0);
-	  if (NULL == info_nis)
-            {
-              info_nis = __pwdalloc();
-              if (NULL == info_nis)
-                return(NULL);
-            }
-          p = __nis_getpwnam(p->pw_name + 1, info_nis);
+          p = __nis_getpwnam(p->pw_name + 1, info);
           if (p && p->pw_uid == uid)
 	    {
 	      __nis_copy_pwd_args(stored_pwd, p, 1);
@@ -445,6 +544,8 @@ _compat_getpwuid(register uid_t uid)
       if (p && p->pw_uid == uid)
         break;
     }
+  if (netgroups)
+    free(netgroups);
   (void) fclose(stream);
   return(p);
 }
@@ -452,7 +553,6 @@ _compat_getpwuid(register uid_t uid)
 static struct passwd *
 __nis_getpwuid(uid_t uid, void *info)
 {
-  static char *nisdomain = NULL;
   char *outkey;
   int status, outkeylen ;
   struct passwd *pwptr = NULL;
@@ -475,41 +575,6 @@ __nis_getpwuid(uid_t uid, void *info)
   return NULL;
 }
 
-static struct passwd *
-__netgroup_getpwuid(const char *netgr, uid_t uid, void *info)
-{
-  int status;
-  static char *nisdomain = NULL;
-  char *host, *user, *domain;
-  struct passwd *p = NULL;
-    
-  if (NULL == nisdomain)
-    yp_get_default_domain(&nisdomain);
-  
-  setnetgrent(netgr);
-  while(1)
-    {
-      while (1 == (status = getnetgrent(&host, &user, &domain))
-             && NULL == user
-             && NULL != domain
-             && 0 != strcmp(domain, nisdomain))
-        ;
-      
-      if (0 == status || NULL == user)
-        {
-          p = NULL;
-          break;
-        }
-      
-      p = __nis_getpwnam(user, info);
-      if (p && p->pw_uid == uid)
-        break;
-      }
-  endnetgrent();
-  return p;
-}
-
-
 static struct passwd *
 __nis_getpwent(int first, void *info)
 {
@@ -605,7 +670,6 @@ __nis_getpwnam(const char *name, void *info)
 static struct passwd *
 __netgroup_getpwent(const char *netgr, int first, void *info)
 {
-  static char *nisdomain = NULL;
   char *host, *user, *domain;
   int status;
   
