@@ -12,9 +12,15 @@ speed_t cfgetispeed(struct termios *tp)
 
 int cfsetospeed(struct termios *tp, speed_t speed)
 {
+#ifdef B460800
+#define _BMAX B460800
+#else
+#define _BMAX B230400
+#endif
+
 #ifdef CBAUDEX
     if ((speed & ~CBAUD) || 
-	((speed & CBAUDEX) && (speed < B57600 || speed > B230400)))
+	((speed & CBAUDEX) && (speed < B57600 || speed > _BMAX)))
 	return 0;
 #else
     if (speed & ~CBAUD)
