@@ -39,11 +39,6 @@ DEFUN(__isinf, (value), double value)
 }
 
 #include <gnu-stabs.h>
-#ifdef __ELF__
 #ifdef weak_alias
 weak_alias (__isinf, isinf);
 #endif
-#else
-function_alias(isinf, __isinf, int, (value),
-	       DEFUN(isinfl, (value), double value));
-#endif
