@@ -30,9 +30,15 @@ DEFUN(__isinfl, (value), long double value)
   union ieee854_double *u = (union ieee854_double *) &value;
 
   /* An IEEE 854 infinity has an exponent with the
-     maximum possible value and a zero mantissa.  */
+     maximum possible value and a mantissa of 0x8000000000000000.
+     For the 68881 the integer bit is dont-care.  */
   if ((u->ieee.exponent & 0x7fff) == 0x7fff &&
-      u->ieee.mantissa0 == 0 && u->ieee.mantissa1 == 0)
+#ifdef __mc68000__
+      (u->ieee.mantissa0 & 0x7fffffff) == 0 &&
+#else
+      u->ieee.mantissa0 == 0x80000000 &&
+#endif
+      u->ieee.mantissa1 == 0)
     return u->ieee.negative ? -1 : 1;
 
   return 0;
