@@ -1,4 +1,10 @@
 #include <syscall.h>
+#include <linux/version.h>
+#ifdef LINUX_VERSION_CODE
+#if LINUX_VERSION_CODE >= 131328
+#include <asm/ipc.h>
+#endif
+#endif
 
 #define __KERNEL__ 
 #undef asmlinkage
